@@ -1,6 +1,7 @@
 # standard library imports
 import io
 import logging
+import shutil
 import struct
 import warnings
 
@@ -10,6 +11,7 @@ from uuid import UUID
 
 # local imports
 from glymur import Jp2k
+from glymur.core import SRGB, RESTRICTED_ICC_PROFILE
 from .lib import tiff as libtiff
 from . import jp2box
 from ._tiff import TAGNUM2NAME
@@ -51,6 +53,8 @@ class Tiff2Jp2k(object):
         If true, then this TIFF must be a GEOTIFF
     tiff_filename : path or str
         Path to TIFF file.
+    jp2 : JP2K object
+        Write to this JPEG2000 file
     jp2_filename : path or str
         Path to JPEG 2000 file to be written.
     jp2_kwargs : dict
@@ -61,12 +65,15 @@ class Tiff2Jp2k(object):
         Create a UUIDBox for the TIFF IFD metadata.
     version : int
         Identifies the TIFF as 32-bit TIFF or 64-bit TIFF.
+    xmp_data : bytes
+        Encoded bytes from XML_PACKET tag (700), or None if not present.
     """
 
     def __init__(
         self, tiff_filename, jp2_filename,
-        create_exif_uuid=True, create_xmp_uuid=False, exclude_tags=None,
-        tilesize=None, verbosity=logging.CRITICAL,
+        create_exif_uuid=True, create_xmp_uuid=True,
+        exclude_tags=None, tilesize=None,
+        verbosity=logging.CRITICAL, include_icc_profile=False,
         **kwargs
     ):
         """
@@ -74,20 +81,23 @@ class Tiff2Jp2k(object):
         ----------
         create_exif_uuid : bool
             If true, create an EXIF UUID out of the TIFF metadata (tags)
+        create_xmp_uuid : bool
+            If true and if there is an XMLPacket (700) tag in the TIFF main
+            IFD, it will be removed from the IFD and placed in a UUID box.
+        include_icc_profile : bool
+            If true, consume any ICC profile tag (34765) into the colour
+            specification box.
         exclude_tags : list or None
             If not None and if create_exif_uuid is True, exclude any listed
             tags from the EXIF UUID.
-        tiff_filename : path or str
-            Path to TIFF file.
         jp2_filename : path or str
             Path to JPEG 2000 file to be written.
+        tiff_filename : path or str
+            Path to TIFF file.
         tilesize : tuple
             The dimensions of a tile in the JP2K file.
         verbosity : int
             Set the level of logging, i.e. WARNING, INFO, etc.
-        create_xmp_uuid : bool
-            If true and if there is an XMLPacket (700) tag in the TIFF main
-            IFD, it will be removed from the IFD and placed in a UUID box.
         """
 
         self.tiff_filename = tiff_filename
@@ -98,11 +108,24 @@ class Tiff2Jp2k(object):
         self.tilesize = tilesize
         self.create_exif_uuid = create_exif_uuid
         self.create_xmp_uuid = create_xmp_uuid
+        self.include_icc_profile = include_icc_profile
 
+        if exclude_tags is None:
+            exclude_tags = []
         self.exclude_tags = self._process_exclude_tags(exclude_tags)
 
+        self.jp2 = None
         self.jp2_kwargs = kwargs
 
+        # Assume that there is no ColorMap tag until we know otherwise.
+        self._colormap = None
+
+        # Assume that there is no ICC profile tag until we know otherwise.
+        self.icc_profile = None
+
+        # Assume no XML_PACKET tag until we know otherwise.
+        self.xmp_data = None
+
         self.setup_logging(verbosity)
 
     def _process_exclude_tags(self, exclude_tags):
@@ -158,9 +181,7 @@ class Tiff2Jp2k(object):
                 # numeric value
                 lst.append(tag_num)
 
-        exclude_tags = lst
-
-        return exclude_tags
+        return lst
 
     def setup_logging(self, verbosity):
         self.logger = logging.getLogger('tiff2jp2')
@@ -170,6 +191,9 @@ class Tiff2Jp2k(object):
         self.logger.addHandler(ch)
 
     def __enter__(self):
+        """
+        The Tiff2Jp2k must be used with a context manager.
+        """
         self.tiff_fp = libtiff.open(self.tiff_filename)
         return self
 
@@ -181,12 +205,101 @@ class Tiff2Jp2k(object):
         self.get_main_ifd()
         self.copy_image()
         self.append_extra_jp2_boxes()
+        self.rewrap_jp2()
+
+    def rewrap_jp2(self):
+        """
+        These re-wrap operations should be mutually exclusive.  An ICC profile
+        should not exist in a TIFF with a colormap.
+        """
+        self.rewrap_for_colormap()
+        self.rewrap_for_icc_profile()
+
+    def rewrap_for_colormap(self):
+        """
+        If the photometric interpretation was PALETTE, then we need to insert
+        a pclr box and a cmap (component mapping box).
+        """
+
+        photo = self.get_tag_value(262)
+        if photo != libtiff.Photometric.PALETTE:
+            return
+
+        jp2h = [box for box in self.jp2.box if box.box_id == 'jp2h'][0]
+
+        bps = (8, 8, 8)
+        pclr = jp2box.PaletteBox(
+            palette=self._colormap,
+            bits_per_component=bps,
+            signed=(False, False, False)
+        )
+        jp2h.box.append(pclr)
+
+        # append the component mapping box
+        cmap = jp2box.ComponentMappingBox(
+            component_index=(0, 0, 0),
+            mapping_type=(1, 1, 1),
+            palette_index=(0, 1, 2)
+        )
+        jp2h.box.append(cmap)
+
+        # fix the colr box.  the colorspace needs to be changed from greyscale
+        # to rgb
+        colr = [box for box in jp2h.box if box.box_id == 'colr'][0]
+        colr.colorspace = SRGB
+
+        temp_filename = str(self.jp2_filename) + '.tmp'
+        self.jp2.wrap(temp_filename, boxes=self.jp2.box)
+        shutil.move(temp_filename, self.jp2_filename)
+        self.jp2.parse()
+
+    def rewrap_for_icc_profile(self):
+        """
+        Consume a TIFF ICC profile, if one is there.
+        """
+        if self.icc_profile is None and self.include_icc_profile:
+            self.logger.warning("No ICC profile was found.")
+
+        if self.icc_profile is None or not self.include_icc_profile:
+            return
+
+        self.logger.info(
+            'Consuming an ICC profile into JP2 color specification box.'
+        )
+
+        colr = jp2box.ColourSpecificationBox(
+            method=RESTRICTED_ICC_PROFILE,
+            precedence=0,
+            icc_profile=self.icc_profile
+        )
+
+        # construct the new set of JP2 boxes, insert the color specification
+        # box with the ICC profile
+        jp2 = Jp2k(self.jp2_filename)
+        boxes = jp2.box
+        boxes[2].box = [boxes[2].box[0], colr]
+
+        # re-wrap the codestream, involves a file copy
+        tmp_filename = str(self.jp2_filename) + '.tmp'
+
+        with open(tmp_filename, mode='wb') as tfile:
+            jp2.wrap(tfile.name, boxes=boxes)
+
+        shutil.move(tmp_filename, self.jp2_filename)
 
     def append_extra_jp2_boxes(self):
         """
         Copy over the TIFF IFD.  Place it in a UUID box.  Append to the JPEG
         2000 file.
         """
+        self.append_exif_uuid_box()
+        self.append_xmp_uuid_box()
+
+    def append_exif_uuid_box(self):
+        """
+        Append an EXIF UUID box onto the end of the JPEG 2000 file.  It will
+        contain metadata from the TIFF IFD.
+        """
         if not self.create_exif_uuid:
             return
 
@@ -198,12 +311,6 @@ class Tiff2Jp2k(object):
         data = struct.pack('<BBHI', 73, 73, 42, 8)
         b.write(data)
 
-        if 700 in self.tags and self.create_xmp_uuid:
-            # remove the XMLPacket data from the IFD dictionary
-            xmp_data = self.tags.pop(700)
-        else:
-            xmp_data = None
-
         self._write_ifd(b, self.tags)
 
         # create the Exif UUID
@@ -224,12 +331,23 @@ class Tiff2Jp2k(object):
         with open(self.jp2_filename, mode='ab') as f:
             uuid_box.write(f)
 
-        if xmp_data is None:
+        self.jp2.finalize(force_parse=True)
+
+    def append_xmp_uuid_box(self):
+        """
+        Append an XMP UUID box onto the end of the JPEG 2000 file if there was
+        an XMP tag in the TIFF IFD.
+        """
+
+        if self.xmp_data is None:
+            return
+
+        if not self.create_xmp_uuid:
             return
 
         # create the XMP UUID
         the_uuid = jp2box.UUID('be7acfcb-97a9-42e8-9c71-999491e3afac')
-        payload = bytes(xmp_data['payload'])
+        payload = bytes(self.xmp_data)
         box_length = len(payload) + 8
         uuid_box = jp2box.UUIDBox(the_uuid, payload, box_length)
         with open(self.jp2_filename, mode='ab') as f:
@@ -248,6 +366,22 @@ class Tiff2Jp2k(object):
 
             self.tags = self.read_ifd(tfp)
 
+            if 320 in self.tags:
+
+                # the TIFF must have PALETTE photometric interpretation
+                data = np.array(self.tags[320]['payload'])
+                self._colormap = data.reshape(len(data) // 3, 3)
+                self._colormap = self._colormap / 65535
+                self._colormap = (self._colormap * 255).astype(np.uint8)
+
+            if 700 in self.tags:
+
+                # XMLPacket
+                self.xmp_data = self.tags[700]['payload']
+
+            else:
+                self.xmp_data = None
+
             if 34665 in self.tags:
                 # we have an EXIF IFD
                 offset = self.tags[34665]['payload'][0]
@@ -256,6 +390,13 @@ class Tiff2Jp2k(object):
 
                 self.tags[34665]['payload'] = exif_ifd
 
+            if 34675 in self.tags:
+                # ICC profile
+                self.icc_profile = bytes(self.tags[34675]['payload'])
+
+            else:
+                self.icc_profile = None
+
     def read_ifd(self, tfp):
         """
         Process either the main IFD or an Exif IFD
@@ -361,33 +502,31 @@ class Tiff2Jp2k(object):
         return tags
 
     def _write_ifd(self, b, tags):
+        """
+        Write the IFD out to the UUIDBox.  We will always write IFDs
+        for 32-bit TIFFs, i.e. 12 byte tags, meaning just 4 bytes within
+        the tag for the tag data
+        """
 
-        # keep this for writing to the UUID, which will always be for 32-bit
-        # TIFFs
         little_tiff_tag_length = 12
+        max_tag_payload_length = 4
 
+        # exclude any unwanted tags
         if self.exclude_tags is not None:
             for tag in self.exclude_tags:
                 if tag in tags:
                     tags.pop(tag)
 
         num_tags = len(tags)
-
         write_buffer = struct.pack('<H', num_tags)
         b.write(write_buffer)
 
         # Ok, so now we have the IFD main body, but following that we have
         # the tag payloads that cannot fit into 4 bytes.
 
-        # the IFD main body in the TIFF.  As it might be big endian, we cannot
-        # just process it as one big chunk.
-
         ifd_start_loc = b.tell()
         after_ifd_position = ifd_start_loc + num_tags * little_tiff_tag_length
 
-        # We write a little-TIFF IFD
-        max_tag_payload_length = 4
-
         for idx, tag in enumerate(tags):
 
             tag_offset = ifd_start_loc + idx * little_tiff_tag_length
@@ -410,25 +549,26 @@ class Tiff2Jp2k(object):
 
                 # write the tag entry to the UUID
                 new_offset = after_ifd_position
-                outbuffer = struct.pack(
+                buffer = struct.pack(
                     '<HHII', tag, dtype, nvalues, new_offset
                 )
-                b.write(outbuffer)
+                b.write(buffer)
 
                 # now write the payload at the outlying position and then come
                 # back to the same position in the file stream
                 cpos = b.tell()
                 b.seek(new_offset)
 
-                out_format = '<' + tag_dtype[dtype]['format'] * nvalues
-                outbuffer = struct.pack(out_format, *payload)
-                b.write(outbuffer)
+                format = '<' + tag_dtype[dtype]['format'] * nvalues
+                buffer = struct.pack(format, *payload)
+                b.write(buffer)
 
                 # keep track of the next position to write out-of-IFD data
                 after_ifd_position = b.tell()
                 b.seek(cpos)
 
             else:
+
                 # the payload DOES fit into the TIFF tag entry
 
                 payload_format = (
@@ -436,11 +576,9 @@ class Tiff2Jp2k(object):
                     * int(max_tag_payload_length / tag_dtype[dtype]['nbytes'])
                 )
 
-                # the payload DOES fit into the UUID tag entry
-
-                # so write it back into the tag entry in the UUID
-                outbuffer = struct.pack('<HHI', tag, dtype, nvalues)
-                b.write(outbuffer)
+                # write the tag metadata
+                buffer = struct.pack('<HHI', tag, dtype, nvalues)
+                b.write(buffer)
 
                 payload_format = tag_dtype[dtype]['format'] * nvalues
 
@@ -451,14 +589,17 @@ class Tiff2Jp2k(object):
 
                 if tag == 34665:
                     # special case for an EXIF IFD
-                    outbuffer = struct.pack('<I', after_ifd_position)
-                    b.write(outbuffer)
+                    buffer = struct.pack('<I', after_ifd_position)
+                    b.write(buffer)
                     b.seek(after_ifd_position)
-                    self._write_ifd(b, payload)
+                    after_ifd_position = self._write_ifd(b, payload)
+
                 else:
                     # write a normal tag
-                    outbuffer = struct.pack('<' + payload_format, *payload)
-                    b.write(outbuffer)
+                    buffer = struct.pack('<' + payload_format, *payload)
+                    b.write(buffer)
+
+        return after_ifd_position
 
     def read_tiff_header(self, tfp):
         """
@@ -518,8 +659,7 @@ class Tiff2Jp2k(object):
 
     def copy_image(self):
         """
-        Transfer the image data from the TIFF to the JPEG 2000 file.  If the
-        TIFF has a stripped configuration, this may be somewhat inefficient.
+        Transfer the image data from the TIFF to the JPEG 2000 file.
         """
 
         if libtiff.isTiled(self.tiff_fp):
@@ -585,15 +725,10 @@ class Tiff2Jp2k(object):
             # Using the RGBA interface is the only reasonable way to deal with
             # this.
             use_rgba_interface = True
-        elif photo == libtiff.Photometric.PALETTE:
-            # Using the RGBA interface is the only reasonable way to deal with
-            # this.  The single plane gets turned into RGB.
-            use_rgba_interface = True
-            spp = 3
         else:
             use_rgba_interface = False
 
-        jp2 = Jp2k(
+        self.jp2 = Jp2k(
             self.jp2_filename,
             shape=(imageheight, imagewidth, spp),
             tilesize=self.tilesize,
@@ -613,9 +748,9 @@ class Tiff2Jp2k(object):
             # this handles both cases of a striped TIFF and a tiled TIFF
 
             self._write_rgba_single_tile(
-                photo, imagewidth, imageheight, spp, jp2
+                    photo, imagewidth, imageheight, spp
             )
-            jp2.finalize(force_parse=True)
+            self.jp2.finalize(force_parse=True)
 
         elif isTiled and self.tilesize is not None:
 
@@ -624,8 +759,7 @@ class Tiff2Jp2k(object):
                 jtw, jth, tw, th,
                 num_jp2k_tile_cols, num_jp2k_tile_rows,
                 dtype,
-                use_rgba_interface,
-                jp2
+                use_rgba_interface
             )
 
         elif not isTiled and self.tilesize is not None:
@@ -635,8 +769,7 @@ class Tiff2Jp2k(object):
                 jtw, jth, rps,
                 num_jp2k_tile_cols, num_jp2k_tile_rows,
                 dtype,
-                use_rgba_interface,
-                jp2
+                use_rgba_interface
             )
 
     def tagvalue2str(self, cls, tag_value):
@@ -652,7 +785,7 @@ class Tiff2Jp2k(object):
         return tag_value_string
 
     def _write_rgba_single_tile(
-        self, photo, imagewidth, imageheight, spp, jp2
+        self, photo, imagewidth, imageheight, spp
     ):
         """
         If no jp2k tiling was specified and if the image is ok to read
@@ -665,8 +798,6 @@ class Tiff2Jp2k(object):
         photo, imagewidth, imageheight, spp : int
             TIFF tag values corresponding to the photometric interpretation,
             image width and height, and samples per pixel
-        jp2 : JP2K object
-            Write to this JPEG2000 file
         """
         msg = (
             "Reading using the RGBA interface, writing as a single tile "
@@ -695,11 +826,11 @@ class Tiff2Jp2k(object):
         if spp < 4:
             image = image[:, :, :3]
 
-        jp2[:] = image
+        self.jp2[:] = image
 
     def _write_tiled_tiff_to_tiled_jp2k(
         self, imagewidth, imageheight, spp, jtw, jth, tw, th,
-        num_jp2k_tile_cols, num_jp2k_tile_rows, dtype, use_rgba_interface, jp2
+        num_jp2k_tile_cols, num_jp2k_tile_rows, dtype, use_rgba_interface
     ):
         """
         The input TIFF image is tiled and we are to create the output JPEG2000
@@ -721,8 +852,6 @@ class Tiff2Jp2k(object):
             Datatype of the image.
         use_rgba_interface : bool
             If true, use the RGBA interface to read the TIFF image data.
-        jp2 : JP2K object
-            Write to this JPEG2000 file
         """
 
         num_tiff_tile_cols = int(np.ceil(imagewidth / tw))
@@ -735,10 +864,16 @@ class Tiff2Jp2k(object):
         self.logger.debug(f'image:  {imageheight} x {imagewidth}')
         self.logger.debug(f'jptile:  {jth} x {jtw}')
         self.logger.debug(f'ttile:  {th} x {tw}')
-        for idx, tilewriter in enumerate(jp2.get_tilewriters()):
+        for idx, tilewriter in enumerate(self.jp2.get_tilewriters()):
 
             # populate the jp2k tile with tiff tiles
-            self.logger.info(f'Tile:  #{idx}')
+            msg = (
+                f'Tile:  #{idx} '
+                f'row #{idx // num_jp2k_tile_cols} '
+                f'col #{idx % num_jp2k_tile_cols} '
+            )
+            self.logger.info(msg)
+
             self.logger.debug(f'J tile row:  #{idx // num_jp2k_tile_cols}')
             self.logger.debug(f'J tile col:  #{idx % num_jp2k_tile_cols}')
 
@@ -840,7 +975,7 @@ class Tiff2Jp2k(object):
 
     def _write_striped_tiff_to_tiled_jp2k(
         self, imagewidth, imageheight, spp, jtw, jth, rps,
-        num_jp2k_tile_cols, num_jp2k_tile_rows, dtype, use_rgba_interface, jp2
+        num_jp2k_tile_cols, num_jp2k_tile_rows, dtype, use_rgba_interface
     ):
         """
         The input TIFF image is striped and we are to create the output
@@ -862,8 +997,6 @@ class Tiff2Jp2k(object):
             Datatype of the image.
         use_rgba_interface : bool
             If true, use the RGBA interface to read the TIFF image data.
-        jp2 : JP2K object
-            Write to this JPEG2000 file
         """
 
         self.logger.debug(f'image:  {imageheight} x {imagewidth}')
@@ -872,13 +1005,18 @@ class Tiff2Jp2k(object):
 
         num_jp2k_tile_cols = int(np.ceil(imagewidth / jtw))
 
-        for idx, tilewriter in enumerate(jp2.get_tilewriters()):
-
-            self.logger.info(f'Tile: #{idx}')
+        for idx, tilewriter in enumerate(self.jp2.get_tilewriters()):
 
             jp2k_tile_row = idx // num_jp2k_tile_cols
             jp2k_tile_col = idx % num_jp2k_tile_cols
 
+            msg = (
+                f'Tile:  #{idx} '
+                f'row #{jp2k_tile_row} '
+                f'col #{jp2k_tile_col}'
+            )
+            self.logger.info(msg)
+
             # the coordinates of the upper left pixel of the jp2k tile
             julr, julc = jp2k_tile_row * jth, jp2k_tile_col * jtw
 
