@@ -975,21 +975,53 @@ class TestPrinting(fixtures.TestCommon):
             with open(self.jp2file, 'rb') as ifptr:
                 tfile.write(ifptr.read())
 
+            b = BytesIO()
+
+            # UUID stuff at byte 0
+            # Exif leader at byte 24
+            # TIFF header at byte 30
+            # IFD start at byte 38
+            # IFD tags at byte 40
+            # tile offsets at byte 88 (40 bytes)
+            #    IFD location 58
+            # Exif IFD start at byte 128
+            #    IFD byte location 98
+            # Exif IFD tag at byte 130 (12 bytes)
+
             # Write L, T, UUID identifier.
-            tfile.write(struct.pack('>I4s', 76, b'uuid'))
-            tfile.write(b'JpgTiffExif->JP2')
+            b.write(struct.pack('>I4s', 142, b'uuid'))
+            b.write(b'JpgTiffExif->JP2')
 
-            tfile.write(b'Exif\x00\x00')
+            b.write(b'Exif\x00\x00')
+
+            # write the tiff header
             xbuffer = struct.pack('<BBHI', 73, 73, 42, 8)
-            tfile.write(xbuffer)
+            b.write(xbuffer)
 
-            # We will write just three tags.
-            tfile.write(struct.pack('<H', 3))
+            # We will write just four tags.
+            b.write(struct.pack('<H', 4))
 
             # The "Make" tag is tag no. 271.
-            tfile.write(struct.pack('<HHII', 256, 4, 1, 256))
-            tfile.write(struct.pack('<HHII', 257, 4, 1, 512))
-            tfile.write(struct.pack('<HHI4s', 271, 2, 3, b'HTC\x00'))
+            b.write(struct.pack('<HHII', 256, 4, 1, 256))
+            b.write(struct.pack('<HHII', 257, 4, 1, 512))
+
+            b.write(struct.pack('<HHII', 324, 4, 10, 58))
+
+            b.write(struct.pack('<HHII', 34665, 4, 1, 98))
+
+            # write the tile offsets (fake)
+            tile_offsets = list(range(0, 100, 10))
+            b.write(struct.pack('<' + 'I' * 10, *tile_offsets))
+
+            # start writing the Exif IFD.
+
+            # We will write just one tag.
+            b.write(struct.pack('<H', 1))
+
+            b.write(struct.pack('<HHI4s', 271, 2, 3, b'HTC\x00'))
+            b.flush()
+
+            tfile.write(b.getvalue())
             tfile.flush()
 
             j = glymur.Jp2k(tfile.name)
@@ -997,10 +1029,14 @@ class TestPrinting(fixtures.TestCommon):
             actual = str(j.box[5])
 
         expected = (
-            "UUID Box (uuid) @ (1135519, 76)\n"
+            "UUID Box (uuid) @ (1135519, 142)\n"
             "    UUID:  4a706754-6966-6645-7869-662d3e4a5032 (EXIF)\n"
-            "    UUID Data:  OrderedDict([('ImageWidth', 256), "
-            "('ImageLength', 512), ('Make', 'HTC')])"
+            "    UUID Data:  OrderedDict([   ('ImageWidth', 256),\n"
+            "                    ('ImageLength', 512),\n"
+            "                    (   'TileOffsets',\n"
+            "                        "
+            "array([ 0, 10, 20, ..., 70, 80, 90], dtype=uint32)),\n"
+            "                    ('ExifTag', OrderedDict([('Make', 'HTC')]))])"
         )
         self.assertEqual(actual, expected)
 
@@ -1162,16 +1198,20 @@ class TestPrinting(fixtures.TestCommon):
 
         Original file tested was input/conformance/p0_15.j2k
         """
-        segment = glymur.codestream.TLMsegment(0,
-                                               (0, 1, 2, 3),
-                                               (4267, 2117, 4080, 2081),
-                                               28, 268)
+        segment = glymur.codestream.TLMsegment(
+            0,
+            (0, 1, 2, 3, 4, 5, 6, 7),
+            (4267, 2117, 4080, 2081, 1000, 100, 150, 400),
+            28, 268
+        )
         actual = str(segment)
 
-        expected = ('TLM marker segment @ (268, 28)\n'
-                    '    Index:  0\n'
-                    '    Tile number:  (0, 1, 2, 3)\n'
-                    '    Length:  (4267, 2117, 4080, 2081)')
+        expected = (
+            'TLM marker segment @ (268, 28)\n'
+            '    Index:  0\n'
+            '    Tile number:  [0 1 2 ... 5 6 7]\n'
+            '    Length:  [4267 2117 4080 ...  100  150  400]'
+        )
 
         self.assertEqual(actual, expected)
 
