@@ -2,6 +2,7 @@
 import importlib.resources as ir
 import logging
 import pathlib
+import platform
 import shutil
 import struct
 import sys
@@ -16,6 +17,7 @@ import numpy as np
 # Local imports
 import glymur
 from glymur import Jp2k, Tiff2Jp2k, command_line
+from glymur.core import SRGB
 from . import fixtures
 from .fixtures import OPENJPEG_NOT_AVAILABLE, OPENJPEG_NOT_AVAILABLE_MSG
 from glymur.lib import tiff as libtiff
@@ -25,126 +27,7 @@ from glymur.lib import tiff as libtiff
     not fixtures.HAVE_SCIKIT_IMAGE, fixtures.HAVE_SCIKIT_IMAGE_MSG
 )
 @unittest.skipIf(OPENJPEG_NOT_AVAILABLE, OPENJPEG_NOT_AVAILABLE_MSG)
-class TestSuite(fixtures.TestCommon):
-
-    @classmethod
-    def setup_exif(cls, path):
-        """
-        Create a simple TIFF file that is constructed to contain an EXIF IFD.
-        """
-
-        with path.open(mode='wb') as f:
-
-            w = 256
-            h = 256
-            rps = 64
-            header_length = 8
-
-            # write the header (8 bytes).  The IFD will follow the image data
-            # (256x256 bytes), so the offset to the IFD will be 8 + h * w.
-            main_ifd_offset = header_length + h * w
-            buffer = struct.pack('<BBHI', 73, 73, 42, main_ifd_offset)
-            f.write(buffer)
-
-            # write the image data, 4 64x256 strips of all zeros
-            strip = bytes([0] * rps * w)
-            f.write(strip)
-            f.write(strip)
-            f.write(strip)
-            f.write(strip)
-
-            # write an IFD with 11 tags
-            main_ifd_data_offset = main_ifd_offset + 2 + 11 * 12 + 4
-
-            buffer = struct.pack('<H', 11)
-            f.write(buffer)
-
-            # width and length and bitspersample
-            buffer = struct.pack('<HHII', 256, 4, 1, w)
-            f.write(buffer)
-            buffer = struct.pack('<HHII', 257, 4, 1, h)
-            f.write(buffer)
-            buffer = struct.pack('<HHII', 258, 4, 1, 8)
-            f.write(buffer)
-
-            # photometric
-            buffer = struct.pack('<HHII', 262, 4, 1, 1)
-            f.write(buffer)
-
-            # strip offsets
-            buffer = struct.pack('<HHII', 273, 4, 4, main_ifd_data_offset)
-            f.write(buffer)
-
-            # spp
-            buffer = struct.pack('<HHII', 277, 4, 1, 1)
-            f.write(buffer)
-
-            # rps
-            buffer = struct.pack('<HHII', 278, 4, 1, 64)
-            f.write(buffer)
-
-            # strip byte counts
-            buffer = struct.pack('<HHII', 279, 4, 4, main_ifd_data_offset + 16)
-            f.write(buffer)
-
-            # pagenumber
-            buffer = struct.pack('<HHIHH', 297, 3, 2, 1, 0)
-            f.write(buffer)
-
-            # XMP
-            with ir.path('tests.data', 'issue555.xmp') as xmp_path:
-                with xmp_path.open() as f2:
-                    xmp = f2.read()
-                    xmp = xmp + '\0'
-            buffer = struct.pack(
-                '<HHII', 700, 1, len(xmp), main_ifd_data_offset + 32
-            )
-            f.write(buffer)
-
-            # exif tag
-            exif_ifd_offset = main_ifd_data_offset + 32 + len(xmp)
-            buffer = struct.pack('<HHII', 34665, 4, 1, exif_ifd_offset)
-            f.write(buffer)
-
-            # terminate the IFD
-            buffer = struct.pack('<I', 0)
-            f.write(buffer)
-
-            # write the strip offsets here
-            buffer = struct.pack(
-                '<IIII', 8, 8 + rps*w, 8 + 2*rps*w, 8 + 3*rps*w
-            )
-            f.write(buffer)
-
-            # write the strip byte counts
-            buffer = struct.pack('<IIII', rps*w, rps*w, rps*w, rps*w)
-            f.write(buffer)
-
-            # write the XMP data
-            f.write(xmp.encode('utf-8'))
-
-            # write a minimal Exif IFD
-            buffer = struct.pack('<H', 2)
-            f.write(buffer)
-
-            # exposure program
-            buffer = struct.pack('<HHIHH', 34850, 3, 1, 2, 0)
-            f.write(buffer)
-
-            # lens model
-            data_location = exif_ifd_offset + 2 + 2*12 + 4
-            buffer = struct.pack('<HHII', 42036, 2, 6, data_location)
-            f.write(buffer)
-
-            # terminate the IFD
-            buffer = struct.pack('<I', 0)
-            f.write(buffer)
-
-            data = 'Canon\0'.encode('utf-8')
-            buffer = struct.pack('<BBBBBB', *data)
-            f.write(buffer)
-
-        cls.exif_tiff = path
+class TestSuiteScikitImage(fixtures.TestCommon):
 
     @classmethod
     def setup_minisblack_spp1(cls, path):
@@ -590,8 +473,6 @@ class TestSuite(fixtures.TestCommon):
         cls.test_tiff_dir = tempfile.mkdtemp()
         cls.test_tiff_path = pathlib.Path(cls.test_tiff_dir)
 
-        cls.setup_exif(cls.test_tiff_path / 'exif.tif')
-
         cls.setup_minisblack_spp1(cls.test_tiff_path / 'moon.tif')
 
         cls.setup_minisblack_3x3(cls.test_tiff_path / 'minisblack_3x3.tif')
@@ -621,43 +502,6 @@ class TestSuite(fixtures.TestCommon):
     def tearDownClass(cls):
         shutil.rmtree(cls.test_tiff_dir)
 
-    def test_exclude_tags_camelcase(self):
-        """
-        Scenario:  Convert TIFF to JP2, but exclude the StripByteCounts and
-        StripOffsets tags.  Supply the argments as camel-case.
-
-        Expected Result:  No warnings, no errors.  The Exif LensModel tag is
-        recoverable from the UUIDbox.
-        """
-        with Tiff2Jp2k(
-            self.exif_tiff, self.temp_jp2_filename,
-            exclude_tags=['StripOffsets', 'StripByteCounts']
-        ) as p:
-            p.run()
-
-        j = Jp2k(self.temp_jp2_filename)
-
-        tags = j.box[-1].data
-        self.assertNotIn('StripByteCounts', tags)
-        self.assertNotIn('StripOffsets', tags)
-
-    def test_exif(self):
-        """
-        Scenario:  Convert TIFF with Exif IFD to JP2
-
-        Expected Result:  No warnings, no errors.  The Exif LensModel tag is
-        recoverable from the UUIDbox.
-        """
-        with Tiff2Jp2k(self.exif_tiff, self.temp_jp2_filename) as p:
-            with warnings.catch_warnings(record=True) as w:
-                p.run()
-                self.assertEqual(len(w), 0)
-
-        j = Jp2k(self.temp_jp2_filename)
-
-        tags = j.box[-1].data
-        self.assertEqual(tags['ExifTag']['LensModel'], 'Canon')
-
     def test_smoke(self):
         """
         SCENARIO:  Convert TIFF file to JP2
@@ -779,27 +623,6 @@ class TestSuite(fixtures.TestCommon):
         self.assertEqual(j.box[-1].data['ImageWidth'], 512)
         self.assertEqual(j.box[-1].data['ImageLength'], 512)
 
-    def test_geotiff(self):
-        """
-        SCENARIO:  Convert a one-component GEOTIFF file to JP2
-
-        EXPECTED RESULT:  there is a geotiff UUID.  The JP2 file has only one
-        component.
-        """
-        with warnings.catch_warnings():
-            warnings.simplefilter('ignore')
-            with ir.path('tests.data', 'albers27.tif') as path:
-                with Tiff2Jp2k(path, self.temp_jp2_filename) as j:
-                    j.run()
-
-        j = Jp2k(self.temp_jp2_filename)
-
-        self.assertEqual(j.box[-1].box_id, 'uuid')
-        self.assertEqual(
-            j.box[-1].uuid, UUID('b14bf8bd-083d-4b43-a5ae-8cd7d5a6ce03')
-        )
-        self.assertEqual(j.box[2].box[0].num_components, 1)
-
     def test_no_uuid(self):
         """
         SCENARIO:  Convert TIFF file to JP2, but do not include the UUID box
@@ -823,6 +646,9 @@ class TestSuite(fixtures.TestCommon):
         )
         self.assertFalse(at_least_one_uuid)
 
+    @unittest.skipIf(
+        platform.machine() == 's390x', 'See issue #546'
+    )
     def test_psnr(self):
         """
         SCENARIO:  Convert TIFF file to JP2 with the irreversible transform.
@@ -1270,47 +1096,6 @@ class TestSuite(fixtures.TestCommon):
         self.assertEqual(c.segment[1].xtsiz, 240)
         self.assertEqual(c.segment[1].ytsiz, 240)
 
-    def test_separated_configuration(self):
-        """
-        SCENARIO:  The TIFF has a planar configuration of SEPARATE which is
-        not supported if a tilesize is specified.
-
-        EXPECTED RESULT:  RuntimeError
-        """
-        with self.assertRaises(RuntimeError):
-            with ir.path(
-                'tests.data', 'flower-separated-planar-08.tif'
-            ) as path:
-                with Tiff2Jp2k(
-                    path, self.temp_jp2_filename, tilesize=(64, 64)
-                ) as j:
-                    j.run()
-
-    def test_bad_tile_size(self):
-        """
-        SCENARIO:  Specify a tilesize that exceeds the image size.  This will
-        cause a segfault unless caught.
-
-        EXPECTED RESULT:  RuntimeError
-        """
-        with self.assertRaises(RuntimeError):
-            with ir.path('tests.data', 'albers27-8.tif') as path:
-                with Tiff2Jp2k(
-                    path, self.temp_jp2_filename, tilesize=(256, 256),
-                ) as j:
-                    j.run()
-
-    def test_minisblack_spp1_bigtiff(self):
-        """
-        SCENARIO:  Convert minisblack BigTIFF file to JP2.  The TIFF has tag
-        XResolution.
-
-        EXPECTED RESULT:  no errors.
-        """
-        with ir.path('tests.data', 'albers27-8.tif') as path:
-            with Tiff2Jp2k(path, self.temp_jp2_filename) as j:
-                j.run()
-
     def test_rgb_tiled_bigtiff(self):
         """
         SCENARIO:  Convert RGB BigTIFF file to JP2.  The TIFF is evenly
@@ -1428,18 +1213,6 @@ class TestSuite(fixtures.TestCommon):
         self.assertEqual(c.segment[1].xtsiz, 512)
         self.assertEqual(c.segment[1].ytsiz, 512)
 
-    def test_tiff_file_not_there(self):
-        """
-        Scenario:  The input TIFF file is not present.
-
-        Expected Result:  FileNotFoundError
-        """
-
-        with self.assertRaises(FileNotFoundError):
-            Tiff2Jp2k(
-                self.test_dir_path / 'not_there.tif', self.temp_jp2_filename
-            )
-
     def test_rgb_uint16(self):
         """
         SCENARIO:  Convert RGB TIFF file to JP2.  The TIFF is evenly
@@ -1570,22 +1343,24 @@ class TestSuite(fixtures.TestCommon):
 
 class TestSuiteNoScikitImage(fixtures.TestCommon):
 
-    @classmethod
-    def setUpClass(cls):
-
-        cls.test_tiff_dir = tempfile.mkdtemp()
-        cls.test_tiff_path = pathlib.Path(cls.test_tiff_dir)
-
-        cls.setup_rgb_evenly_stripped(cls.test_tiff_path / 'goodstuff.tif')
-
-        cls.setup_exif(cls.test_tiff_path / 'exif.tif')
-
     @classmethod
     def setup_exif(cls, path):
         """
         Create a simple TIFF file that is constructed to contain an EXIF IFD.
         """
 
+        # main TIFF header @ 0
+        # image data @ 8
+        # main IFD @ 65544 = 256*256 + 8 (2 + 12*12 = 146 bytes)
+        # main IDF data @ 65690  = main_ifd + 2 + 12 * 12 + 4
+        #
+        # strip offsets @ 65694 (16 bytes)
+        # strip byte counts @ 65710 (16 bytes)
+        # xmp data @ 65726 (12532 bytes)
+        # camera ID data @ 78258 (8 bytes)
+        #
+        # exif IFD @ 78266 (2 + 2*12 + 4 = 30 bytes)
+        # exif IFD data @ 78296 (6 bytes)
         with path.open(mode='wb') as f:
 
             w = 256
@@ -1606,10 +1381,10 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
             f.write(strip)
             f.write(strip)
 
-            # write an IFD with 11 tags
-            main_ifd_data_offset = main_ifd_offset + 2 + 11 * 12 + 4
+            # write an IFD with 12 tags
+            main_ifd_data_offset = main_ifd_offset + 2 + 12 * 12 + 4
 
-            buffer = struct.pack('<H', 11)
+            buffer = struct.pack('<H', 12)
             f.write(buffer)
 
             # width and length and bitspersample
@@ -1655,10 +1430,16 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
             f.write(buffer)
 
             # exif tag
-            exif_ifd_offset = main_ifd_data_offset + 32 + len(xmp)
+            # write it AFTER lensinfo, which is 8 chars
+            exif_ifd_offset = main_ifd_data_offset + 32 + len(xmp) + 8
             buffer = struct.pack('<HHII', 34665, 4, 1, exif_ifd_offset)
             f.write(buffer)
 
+            # lensmodel
+            offset = main_ifd_data_offset + 32 + len(xmp)
+            buffer = struct.pack('<HHII', 50708, 2, 8, offset)
+            f.write(buffer)
+
             # terminate the IFD
             buffer = struct.pack('<I', 0)
             f.write(buffer)
@@ -1676,6 +1457,9 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
             # write the XMP data
             f.write(xmp.encode('utf-8'))
 
+            # write the camera ID
+            f.write("abcdefg\x00".encode('utf-8'))
+
             # write a minimal Exif IFD
             buffer = struct.pack('<H', 2)
             f.write(buffer)
@@ -1699,6 +1483,16 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
 
         cls.exif_tiff = path
 
+    @classmethod
+    def setUpClass(cls):
+
+        cls.test_tiff_dir = tempfile.mkdtemp()
+        cls.test_tiff_path = pathlib.Path(cls.test_tiff_dir)
+
+        cls.setup_rgb_evenly_stripped(cls.test_tiff_path / 'goodstuff.tif')
+
+        cls.setup_exif(cls.test_tiff_path / 'exif.tif')
+
     @classmethod
     def setup_rgb_evenly_stripped(cls, path):
         """
@@ -1730,6 +1524,60 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
         cls.goodstuff_data = data
         cls.goodstuff_path = path
 
+    def test_numeric_exclude_keyword_argument(self):
+        """
+        Scenario:  specify exclude_tags keyword argument as list of integer
+        keyword argument is set to True.
+
+        Expected result:  The tags are not included in the exif IFD.
+        """
+        with Tiff2Jp2k(
+            self.goodstuff_path, self.temp_jp2_filename,
+            exclude_tags=[273, 279]
+        ) as p:
+            p.run()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        self.assertNotIn('StripOffsets', j.box[-1].data)
+        self.assertNotIn('StripByteCounts', j.box[-1].data)
+
+    def test_string_exclude_keyword_argument(self):
+        """
+        Scenario:  specify exclude_tags keyword argument as list of integer
+        keyword argument is set to True.
+
+        Expected result:  The tags are not included in the exif IFD.
+        """
+        with Tiff2Jp2k(
+            self.goodstuff_path, self.temp_jp2_filename,
+            exclude_tags=['StripOffsets', 'StripByteCounts']
+        ) as p:
+            p.run()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        self.assertNotIn('StripOffsets', j.box[-1].data)
+        self.assertNotIn('StripByteCounts', j.box[-1].data)
+
+    def test_tiff_has_no_icc_profile(self):
+        """
+        Scenario:  input TIFF has no ICC profile, yet the include_icc_profile
+        keyword argument is set to True.
+
+        Expected result:  a warning is issued
+        """
+        with Tiff2Jp2k(
+            self.goodstuff_path, self.temp_jp2_filename, tilesize=(64, 64),
+            include_icc_profile=True, verbosity=logging.INFO
+        ) as j:
+            with self.assertLogs(
+                logger='tiff2jp2', level=logging.WARNING
+            ) as cm:
+                j.run()
+
+                self.assertEqual(len(cm.output), 1)
+
     def test_stripped_logging(self):
         """
         Scenario:  input TIFF is organized by strips and logging is turned on.
@@ -1798,8 +1646,8 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
         Scenario:  Convert TIFF to JP2, but exclude the StripByteCounts and
         StripOffsets tags.
 
-        Expected Result:  No warnings, no errors.  The Exif LensModel tag is
-        recoverable from the UUIDbox.
+        Expected Result:  The Exif UUID box prints without error.
+        The StripByteCounts and StripOffsets tags are not present.
         """
         with Tiff2Jp2k(
             self.exif_tiff, self.temp_jp2_filename,
@@ -1809,7 +1657,7 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
 
         j = Jp2k(self.temp_jp2_filename)
 
-        tags = j.box[-1].data
+        tags = j.box[-2].data
         self.assertNotIn('StripByteCounts', tags)
         self.assertNotIn('StripOffsets', tags)
 
@@ -1832,7 +1680,7 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
 
         j = Jp2k(self.temp_jp2_filename)
 
-        tags = j.box[-1].data
+        tags = j.box[-2].data
         self.assertNotIn('StripByteCounts', tags)
         self.assertNotIn('StripOffsets', tags)
 
@@ -1852,7 +1700,7 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
 
         j = Jp2k(self.temp_jp2_filename)
 
-        tags = j.box[-1].data
+        tags = j.box[-2].data
         self.assertNotIn('StripByteCounts', tags)
         self.assertNotIn('StripOffsets', tags)
 
@@ -1870,7 +1718,9 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
 
         j = Jp2k(self.temp_jp2_filename)
 
-        tags = j.box[-1].data
+        tags = j.box[-2].data
+
+        self.assertEqual(tags['UniqueCameraModel'], 'abcdefg')
         self.assertEqual(tags['ExifTag']['LensModel'], 'Canon')
 
         str(j.box[-1])
@@ -1878,11 +1728,11 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
     def test_xmp(self):
         """
         Scenario:  Convert TIFF with Exif IFD to JP2.  The main IFD has an
-        XML Packet tag (700).  Supply the 'xmp_uuid' keyword.
+        XML Packet tag (700).  Supply the 'xmp_uuid' keyword as True.
 
-        Expected Result:  The XMLPacket tag is removed from the main IFD.
-        An Exif UUID is appended to the end of the JP2 file, and then an XMP
-        UUID is appended.
+        Expected Result:  An Exif UUID is appended to the end of the
+        JP2 file, and then an XMP UUID is appended.  The XMLPacket tag is still
+        present in the UUID IFD.
         """
         with Tiff2Jp2k(
             self.exif_tiff, self.temp_jp2_filename, create_xmp_uuid=True
@@ -1891,12 +1741,69 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
 
         j = Jp2k(self.temp_jp2_filename)
 
+        # first we find the Exif UUID, then maybe the XMP UUID.  The Exif UUID
+        # data should still have have the XMLPacket tag as only the exclude
+        # tags keyword argument can do that.
+        box = j.box[-2]
+        actual = box.uuid
+        expected = UUID(bytes=b'JpgTiffExif->JP2')
+        self.assertEqual(actual, expected)
+        self.assertIn('XMLPacket', box.data)
+
+        # ok so the xmp UUID is the last box
+        xmp_box = j.box[-1]
+        actual = xmp_box.uuid
+        expected = UUID('be7acfcb-97a9-42e8-9c71-999491e3afac')
+        self.assertEqual(actual, expected)
+        self.assertEqual(
+            xmp_box.data.getroot().values(), ['Public XMP Toolkit Core 3.5']
+        )
+
+    def test_xmp_false(self):
+        """
+        Scenario:  Convert TIFF with Exif IFD to JP2.  The main IFD has an
+        XML Packet tag (700).  Supply the 'xmp_uuid' keyword as False.
+
+        Expected Result:  An Exif UUID is appended to the end of the
+        JP2 file, but no XMP UUID is appended.  The XMLPacket tag is still
+        present in the UUID data.
+        """
+        with Tiff2Jp2k(
+            self.exif_tiff, self.temp_jp2_filename, create_xmp_uuid=False
+        ) as p:
+            p.run()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        # we find the Exif UUID at the end.
+        box = j.box[-1]
+        actual = box.uuid
+        expected = UUID(bytes=b'JpgTiffExif->JP2')
+        self.assertEqual(actual, expected)
+        self.assertIn('XMLPacket', box.data)
+
+    def test_xmp__exclude_XMLPacket(self):
+        """
+        Scenario:  Convert TIFF with Exif IFD to JP2.  The main IFD has an
+        XML Packet tag (700).  Supply the 'create_xmp_uuid' keyword.  Supply
+        the exclude_tags keyword, but don't supply XMLPacket.
+
+        Expected Result:  The XMLPacket tag is not removed from the main IFD.
+        An Exif UUID is appended to the end of the JP2 file, and then an XMP
+        UUID is appended.
+        """
+        kwargs = {'create_xmp_uuid': True, 'exclude_tags': ['StripOffsets']}
+        with Tiff2Jp2k(self.exif_tiff, self.temp_jp2_filename, **kwargs) as p:
+            p.run()
+
+        j = Jp2k(self.temp_jp2_filename)
+
         # first we find the Exif UUID, then the XMP UUID.  The Exif UUID
         # data should not have the XMLPacket tag.
         actual = j.box[-2].uuid
         expected = UUID(bytes=b'JpgTiffExif->JP2')
         self.assertEqual(actual, expected)
-        self.assertNotIn('XMLPacket', j.box[-2].data)
+        self.assertIn('XMLPacket', j.box[-2].data)
 
         actual = j.box[-1].uuid
         expected = UUID('be7acfcb-97a9-42e8-9c71-999491e3afac')
@@ -1905,7 +1812,7 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
             j.box[-1].data.getroot().values(), ['Public XMP Toolkit Core 3.5']
         )
 
-    def test_commandline__capture_display_resolution__no_tilesize(self):
+    def test_commandline_capture_display_resolution(self):
         """
         Scenario:  patch sys such that we can run the command
         line tiff2jp2 script.  Supply the --capture-resolution and
@@ -1979,9 +1886,8 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
         Scenario:  patch sys such that we can run the command line tiff2jp2
         script.  Use the --create-xmp-uuid option.
 
-        Expected Result:  The XMLPacket tag is removed from the main IFD.
-        An Exif UUID is appended to the end of the JP2 file, and then an XMP
-        UUID is appended.
+        Expected Result:  An Exif UUID is appended to the end of the
+        JP2 file, and then an XMP UUID is appended.
         """
         sys.argv = [
             '', str(self.exif_tiff), str(self.temp_jp2_filename),
@@ -1992,12 +1898,10 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
 
         j = Jp2k(self.temp_jp2_filename)
 
-        # first we find the Exif UUID, then the XMP UUID.  The Exif UUID
-        # data should not have the XMLPacket tag.
+        # first we find the Exif UUID, then the XMP UUID.
         actual = j.box[-2].uuid
         expected = UUID(bytes=b'JpgTiffExif->JP2')
         self.assertEqual(actual, expected)
-        self.assertNotIn('XMLPacket', j.box[-2].data)
 
         actual = j.box[-1].uuid
         expected = UUID('be7acfcb-97a9-42e8-9c71-999491e3afac')
@@ -2035,6 +1939,123 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
 
         Jp2k(self.temp_jp2_filename)
 
+    def test_icc_profile(self):
+        """
+        Scenario:  The input TIFF has the ICC profile tag.  Provide the
+        include_icc_profile keyword as True.
+
+        Expected Result.  The ICC profile is verified in the
+        ColourSpecificationBox.  There is a logging message at the info
+        level stating that a color profile was consumed.
+        """
+        with ir.path('tests.data', 'basn6a08.tif') as path:
+
+            with path.open(mode='rb') as f:
+                buffer = f.read()
+                ifd = glymur._tiff.tiff_header(buffer)
+            icc_profile = bytes(ifd['ICCProfile'])
+
+            with Tiff2Jp2k(
+                path, self.temp_jp2_filename, include_icc_profile=True
+            ) as p:
+
+                with self.assertLogs(
+                    logger='tiff2jp2', level=logging.INFO
+                ) as cm:
+                    p.run()
+
+                self.assertEqual(
+                    sum('ICC profile' in msg for msg in cm.output), 1
+                )
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        # The colour specification box has the profile
+        self.assertEqual(j.box[2].box[1].icc_profile, bytes(icc_profile))
+
+    def test_icc_profile_commandline(self):
+        """
+        Scenario:  The input TIFF has the ICC profile tag.  Provide the
+        --include-icc-profile argument.
+
+        Expected Result.  The ICC profile is verified in the
+        ColourSpecificationBox.
+        """
+        with ir.path('tests.data', 'basn6a08.tif') as path:
+
+            with path.open(mode='rb') as f:
+                buffer = f.read()
+                ifd = glymur._tiff.tiff_header(buffer)
+            icc_profile = bytes(ifd['ICCProfile'])
+
+            sys.argv = [
+                '', str(path), str(self.temp_jp2_filename),
+                '--include-icc-profile'
+            ]
+            command_line.tiff2jp2()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        # The colour specification box has the profile
+        self.assertEqual(j.box[2].box[1].icc_profile, bytes(icc_profile))
+
+    def test_exclude_icc_profile_commandline(self):
+        """
+        Scenario:  The input TIFF has the ICC profile tag.  Do not provide the
+        --include-icc-profile flag.
+
+        Expected Result.  The ColourSpecificationBox is normal (no ICC
+        profile).  The ICC profile tag will be present in the
+        JpgTiffExif->JP2 UUID box.
+        """
+        with ir.path('tests.data', 'basn6a08.tif') as path:
+
+            sys.argv = [
+                '', str(path), str(self.temp_jp2_filename),
+            ]
+            command_line.tiff2jp2()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        # The colour specification box does not have the profile
+        colr = j.box[2].box[1]
+        self.assertEqual(colr.method, glymur.core.ENUMERATED_COLORSPACE)
+        self.assertEqual(colr.precedence, 0)
+        self.assertEqual(colr.approximation, 0)
+        self.assertEqual(colr.colorspace, SRGB)
+        self.assertIsNone(colr.icc_profile)
+
+    def test_exclude_icc_profile_commandline__exclude_from_uuid(self):
+        """
+        Scenario:  The input TIFF has the ICC profile tag.  Do not specify
+        the --include-icc-profile flag.  Specify the 34675 (ICCProfile) tag
+        in the --exclude-tags flag.
+
+        Expected Result.  The ICC profile is verified to not be present in the
+        ColourSpecificationBox.  The ICC profile tag will be not present in the
+        JpgTiffExif->JP2 UUID box.
+        """
+        with ir.path('tests.data', 'basn6a08.tif') as path:
+
+            sys.argv = [
+                '', str(path), str(self.temp_jp2_filename),
+                '--exclude-tags', 'ICCProfile',
+            ]
+            command_line.tiff2jp2()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        # The colour specification box does not have the profile
+        colr = j.box[2].box[1]
+        self.assertEqual(colr.method, glymur.core.ENUMERATED_COLORSPACE)
+        self.assertEqual(colr.precedence, 0)
+        self.assertEqual(colr.approximation, 0)
+        self.assertEqual(colr.colorspace, SRGB)
+        self.assertIsNone(colr.icc_profile)
+
+        # the exif UUID box does not have the profile
+        self.assertNotIn('ICCProfile', j.box[-1].data)
+
     def test_not_a_tiff(self):
         """
         Scenario:  The input "TIFF" is not actually a TIFF.  This used to
@@ -2046,3 +2067,150 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
             with ir.path('tests.data', 'simple_rdf.txt') as path:
                 with Tiff2Jp2k(path, self.temp_jp2_filename):
                     pass
+
+    def test_colormap(self):
+        """
+        Scenario:  The input "TIFF" has a colormap tag.
+
+        Expected Result:  The output JP2 has a single layer and the jp2h box
+        has a pclr box.
+        """
+        for tag in ['ColorMap', 'StripOffsets']:
+            with self.subTest(tag=tag):
+                self._test_colormap(tag=tag)
+
+    def _test_colormap(self, tag):
+
+        kwargs = {'tilesize': (32, 32), 'exclude_tags': [tag]}
+        with ir.path('tests.data', 'issue572.tif') as path:
+            with Tiff2Jp2k(path, self.temp_jp2_filename, **kwargs) as p:
+                p.run()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        # the image header box shows just a single layer
+        shape = (
+            j.box[2].box[0].height,
+            j.box[2].box[0].width,
+            j.box[2].box[0].num_components,
+        )
+        self.assertEqual(shape, (64, 64, 1))
+
+        # the colr box says sRGB, not greyscale
+        self.assertEqual(j.box[2].box[1].colorspace, SRGB)
+
+        # a pclr box exists
+        self.assertEqual(j.box[2].box[2].box_id, 'pclr')
+
+        # a component mapping box exists
+        self.assertEqual(j.box[2].box[3].box_id, 'cmap')
+        self.assertEqual(j.box[2].box[3].component_index, (0, 0, 0))
+        self.assertEqual(j.box[2].box[3].mapping_type, (1, 1, 1))
+        self.assertEqual(j.box[2].box[3].palette_index, (0, 1, 2))
+
+        # The last box should be the exif uuid.  It may or may not have the
+        # colormap tag depending on what was specified.
+        exif_box = j.box[-1]
+        actual = exif_box.uuid
+        expected = UUID(bytes=b'JpgTiffExif->JP2')
+        self.assertEqual(actual, expected)
+        if tag == 'ColorMap':
+            self.assertNotIn('ColorMap', exif_box.data)
+        else:
+            self.assertIn('ColorMap', exif_box.data)
+
+    def test_excluded_tags_is_none(self):
+        """
+        Scenario:  Convert TIFF to JP2, but provide None for the exclude_tags
+        argument.
+
+        Expected Result:  The UUIDbox has StripOffsets, StripByteCounts, and
+        ICCProfile.
+        """
+        with ir.path('tests.data', 'basn6a08.tif') as path:
+            with Tiff2Jp2k(
+                path, self.temp_jp2_filename, exclude_tags=None
+            ) as p:
+                p.run()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        # last box is exif
+        tags = j.box[-1].data
+        self.assertIn('StripByteCounts', tags)
+        self.assertIn('StripOffsets', tags)
+        self.assertIn('ICCProfile', tags)
+
+    def test_geotiff(self):
+        """
+        SCENARIO:  Convert a one-component GEOTIFF file to JP2
+
+        EXPECTED RESULT:  there is a geotiff UUID.  The JP2 file has only one
+        component.
+        """
+        with warnings.catch_warnings():
+            warnings.simplefilter('ignore')
+            with ir.path('tests.data', 'albers27.tif') as path:
+                with Tiff2Jp2k(path, self.temp_jp2_filename) as j:
+                    j.run()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        self.assertEqual(j.box[-1].box_id, 'uuid')
+        self.assertEqual(
+            j.box[-1].uuid, UUID('b14bf8bd-083d-4b43-a5ae-8cd7d5a6ce03')
+        )
+        self.assertEqual(j.box[2].box[0].num_components, 1)
+
+    def test_separated_configuration(self):
+        """
+        SCENARIO:  The TIFF has a planar configuration of SEPARATE which is
+        not supported if a tilesize is specified.
+
+        EXPECTED RESULT:  RuntimeError
+        """
+        with self.assertRaises(RuntimeError):
+            with ir.path(
+                'tests.data', 'flower-separated-planar-08.tif'
+            ) as path:
+                with Tiff2Jp2k(
+                    path, self.temp_jp2_filename, tilesize=(64, 64)
+                ) as j:
+                    j.run()
+
+    def test_bad_tile_size(self):
+        """
+        SCENARIO:  Specify a tilesize that exceeds the image size.  This will
+        cause a segfault unless caught.
+
+        EXPECTED RESULT:  RuntimeError
+        """
+        with self.assertRaises(RuntimeError):
+            with ir.path('tests.data', 'albers27-8.tif') as path:
+                with Tiff2Jp2k(
+                    path, self.temp_jp2_filename, tilesize=(256, 256),
+                ) as j:
+                    j.run()
+
+    def test_minisblack_spp1_bigtiff(self):
+        """
+        SCENARIO:  Convert minisblack BigTIFF file to JP2.  The TIFF has tag
+        XResolution.
+
+        EXPECTED RESULT:  no errors.
+        """
+        with ir.path('tests.data', 'albers27-8.tif') as path:
+            with Tiff2Jp2k(path, self.temp_jp2_filename) as j:
+                j.run()
+
+    def test_tiff_file_not_there(self):
+        """
+        Scenario:  The input TIFF file is not present.
+
+        Expected Result:  FileNotFoundError
+        """
+
+        with self.assertRaises(FileNotFoundError):
+            Tiff2Jp2k(
+                self.test_dir_path / 'not_there.tif', self.temp_jp2_filename
+            )
